@@ -7,21 +7,20 @@ from copy import deepcopy
 from django.template import Parser, Lexer, Context, TemplateSyntaxError
 
 def run_shell(context=Context()):
-    goodbye = lambda: sys.stderr.write('\nkthxbai!\n')
+    parser = Parser([])
     while True:
         try:
-            for node in input_node_generator():
+            for node in input_node_generator(parser=parser):
                 sys.stdout.write(node.render(context))
-        except KeyboardInterrupt:
-            goodbye()
-            break
-        except EOFError:
-            goodbye()
+        except (KeyboardInterrupt, EOFError):
             break
         except Exception, e:
             traceback.print_exc()
+    sys.stderr.write('\nkthxbai!\n')
 
-def input_node_generator(prompt='>>> ', leading_tokens=None, input_source=raw_input):
+def input_node_generator(prompt='>>> ', leading_tokens=None, input_source=raw_input, parser=None):
+    if parser is None:
+        parser = Parser([])
     input = False
     while not input:
         input = input_source(prompt)
@@ -31,11 +30,12 @@ def input_node_generator(prompt='>>> ', leading_tokens=None, input_source=raw_in
         tokens = leading_tokens + tokens
     initial_tokens = deepcopy(tokens)
     try:
-        for node in Parser(tokens).parse():
+        parser.tokens = tokens
+        for node in parser.parse():
             yield node
     except TemplateSyntaxError, e:
         if e.args[0].startswith('Unclosed tags'):
-            for node in input_node_generator('... ', initial_tokens, input_source):
+            for node in input_node_generator('... ', initial_tokens, input_source, parser):
                 yield node
         else:
             raise
